﻿using System.Collections.Generic;
using DataAccess.Interface;
using NHibernate;
using NHibernate.Criterion;

namespace DataAccess.Dao
{
    public class DaoBase<T> : IDaoBase<T> where T : class, IEntity
    {
        protected ISession Session;

        protected DaoBase()
        {
            Session = NHibernateHelper.Session;
        }

        public IList<T> GetAll()
        {
            return Session.QueryOver<T>().List<T>();
        }

        public object Create(T entity)
        {
            object o;

            using (ITransaction transaction = Session.BeginTransaction())
            {
                o = Session.Save(entity);
                transaction.Commit();
            }

            return o;
        }

        public void Update(T entity)
        {
            using (ITransaction transaction = Session.BeginTransaction())
            {
                Session.Update(entity);
                transaction.Commit();
            }
        }

        public void SaveOrUpdate(T entity)
        {
            using (ITransaction transaction = Session.BeginTransaction())
            {
                Session.SaveOrUpdate(entity);
                transaction.Commit();
            }
        }

        public void Save(T entity)
        {
            using (ITransaction transaction = Session.BeginTransaction())
            {
                Session.Save(entity);
                transaction.Commit();
            }
        }

        public void Merge(T entity)
        {
            using (ITransaction transaction = Session.BeginTransaction())
            {
                Session.Merge(entity);
                transaction.Commit();
            }
        }

        public void Delete(T entity)
        {
            using (ITransaction transaction = Session.BeginTransaction())
            {
                Session.Delete(entity);
                transaction.Commit();
            }
        }

        public T GetById(int id)
        {
            return Session.CreateCriteria<T>().Add(Restrictions.Eq("Id", id)).UniqueResult<T>();
        }
    }
}
