﻿using System;
using System.Collections.Generic;
using System.Linq;
using DataAccess.Model;
using NHibernate.Criterion;

namespace DataAccess.Dao
{
    public class TestDao : DaoBase<Test>
    {
        public IList<Test> GetTestsPaged(int count, int page, out int totalItems)
        {
            totalItems = Session.CreateCriteria<Test>() // vytvoreni kriteria
                .SetProjection(Projections.RowCount()) // spocitani poctu radku
                .UniqueResult<int>(); //vraceni unikatniho vysledku

            return Session.CreateCriteria<Test>().CreateAlias("User", "us")
                .AddOrder(Order.Asc("us.Surname"))
                .SetFirstResult((page - 1) * count)
                .SetMaxResults(count).List<Test>();
        }

        public IList<Test> GetTestsByOrderPaged(int count, int page, out int totalItems, string order)
        {
            totalItems = Session.CreateCriteria<Test>() // vytvoreni kriteria
            .SetProjection(Projections.RowCount()) // spocitani poctu radku
            .UniqueResult<int>(); //vraceni unikatniho vysledku

            switch (order)
            {
                case "upGrade":
                    return Session.CreateCriteria<Test>()
                        .AddOrder(Order.Asc("Grade"))
                        .SetFirstResult((page - 1) * count)
                        .SetMaxResults(count).List<Test>();
                case "downGrade":
                    return Session.CreateCriteria<Test>()
                        .AddOrder(Order.Desc("Grade"))
                        .SetFirstResult((page - 1) * count)
                        .SetMaxResults(count).List<Test>();
                case "upDate":
                    return Session.CreateCriteria<Test>()
                        .AddOrder(Order.Desc("ExecutionDate"))
                        .SetFirstResult((page - 1) * count)
                        .SetMaxResults(count).List<Test>();
                case "downDate":
                    return Session.CreateCriteria<Test>()
                        .AddOrder(Order.Asc("ExecutionDate"))
                        .SetFirstResult((page - 1) * count)
                        .SetMaxResults(count).List<Test>();
                case "upSurname":
                    return Session.CreateCriteria<Test>().CreateAlias("User", "us")
                        .AddOrder(Order.Asc("us.Surname"))
                        .SetFirstResult((page - 1) * count)
                        .SetMaxResults(count).List<Test>();
                case "downSurname":
                    return Session.CreateCriteria<Test>().CreateAlias("User", "us")
                        .AddOrder(Order.Desc("us.Surname"))
                        .SetFirstResult((page - 1) * count)
                        .SetMaxResults(count).List<Test>();
                default:
                    return Session.CreateCriteria<Test>()
                        .AddOrder(Order.Asc("Grade"))
                        .SetFirstResult((page - 1) * count)
                        .SetMaxResults(count).List<Test>();
            }
        }

        public IList<Test> GetByChapter(Chapter chapter)
        {
            return Session.CreateCriteria<Test>()
                .Add(Restrictions.Eq("Chapter", chapter))
                .List<Test>();
        }

        public IList<Test> GetByUser(User user)
        {
            return Session.CreateCriteria<Test>()
                .Add(Restrictions.Eq("User", user))
                .List<Test>();
        }

        public IList<Test> GetByChapterAndUser(User user, Chapter chapter)
        {
            return Session.CreateCriteria<Test>()
                .Add(Restrictions.Eq("User", user)).Add(Restrictions.Eq("Chapter", chapter))
                .List<Test>();
        }

        public Test GetByIdAndUser(int id, User user)
        {
            return Session.CreateCriteria<Test>()
                .Add(Restrictions.Eq("User", user)).Add(Restrictions.Eq("Id", id))
                .UniqueResult<Test>();
        }

        public Tuple<float, int> GetResultForUserByChapter(User user, Chapter chapter)
        {
            IList<Test> tests = Session.CreateCriteria<Test>().Add(Restrictions.Eq("User", user)).Add(Restrictions.Eq("Chapter", chapter))
            .List<Test>();

            float result = tests.Aggregate<Test, float>(0, (current, test) => current + test.Grade);

            return Tuple.Create(result / tests.Count, tests.Count);
        }

        public IList<Test> GetByUserPaged(User user, int count, int page, out int totalItems)
        {
            totalItems = Session.CreateCriteria<Test>().Add(Restrictions.Eq("User", user)) // vytvoreni kriteria
                .SetProjection(Projections.RowCount()) // spocitani poctu radku
                .UniqueResult<int>(); //vraceni unikatniho vysledku

            return Session.CreateCriteria<Test>().Add(Restrictions.Eq("User", user))
                .AddOrder(Order.Desc("ExecutionDate"))
                .SetFirstResult((page - 1) * count)
                .SetMaxResults(count).List<Test>();
        }

        public IList<Test> GetTestsByUserAndOrderPaged(int count, int page, out int totalItems, string order, User user)
        {
            totalItems = Session.CreateCriteria<Test>().Add(Restrictions.Eq("User", user)) // vytvoreni kriteria
            .SetProjection(Projections.RowCount()) // spocitani poctu radku
            .UniqueResult<int>(); //vraceni unikatniho vysledku

            switch (order)
            {
                case "upGrade":
                    return Session.CreateCriteria<Test>().Add(Restrictions.Eq("User", user))
                        .AddOrder(Order.Asc("Grade"))
                        .SetFirstResult((page - 1) * count)
                        .SetMaxResults(count).List<Test>();
                case "downGrade":
                    return Session.CreateCriteria<Test>().Add(Restrictions.Eq("User", user))
                        .AddOrder(Order.Desc("Grade"))
                        .SetFirstResult((page - 1) * count)
                        .SetMaxResults(count).List<Test>();
                case "upDate":
                    return Session.CreateCriteria<Test>().Add(Restrictions.Eq("User", user))
                        .AddOrder(Order.Desc("ExecutionDate"))
                        .SetFirstResult((page - 1) * count)
                        .SetMaxResults(count).List<Test>();
                case "downDate":
                    return Session.CreateCriteria<Test>().Add(Restrictions.Eq("User", user))
                        .AddOrder(Order.Asc("ExecutionDate"))
                        .SetFirstResult((page - 1) * count)
                        .SetMaxResults(count).List<Test>();
                default:
                    return Session.CreateCriteria<Test>().Add(Restrictions.Eq("User", user))
                        .AddOrder(Order.Asc("Grade"))
                        .SetFirstResult((page - 1) * count)
                        .SetMaxResults(count).List<Test>();
            }
        }

        public IList<Test> SearchTests(string phrase)
        {
            return Session.CreateCriteria<Test>().CreateAlias("Chapter", "chap").Add(Restrictions.Like("chap.Name", $"%{phrase}%"))
                .List<Test>();
        }

        public IList<Test> SearchTestsByUser(string phrase, User user)
        {
            return Session.CreateCriteria<Test>().Add(Restrictions.Eq("User", user)).CreateAlias("Chapter", "chap").Add(Restrictions.Like("chap.Name", $"%{phrase}%"))
                .List<Test>();
        }
    }
}
