﻿using System.Collections.Generic;
using System.Linq;
using DataAccess.Model;
using NHibernate.Criterion;

namespace DataAccess.Dao
{
    public class UserDao : DaoBase<User>
    {
        public User GetByLoginAndPassword(string login, string password)
        {
            return Session.CreateCriteria<User>()
                 .Add(Restrictions.Eq("Login", login))
                 .Add(Restrictions.Eq("Password", password))
                 .UniqueResult<User>();
        }

        public User GetByLogin(string login)
        {
            return Session.CreateCriteria<User>()
                 .Add(Restrictions.Eq("Login", login))
                 .UniqueResult<User>();
        }

        public User GetByEmail(string email)
        {
            return Session.CreateCriteria<User>()
                 .Add(Restrictions.Eq("Email", email))
                 .UniqueResult<User>();
        }

        public IList<User> GetByRole(Role role)
        {
            return Session.CreateCriteria<User>()
                 .Add(Restrictions.Eq("Role", role))
                 .List<User>();
        }

        public IList<User> GetAllDeactivated()
        {
            return Session.CreateCriteria<User>()
                 .Add(Restrictions.Not(Restrictions.Eq("State", 1)))
                 .List<User>();
        }

        public IList<User> GetAllRegistered()
        {
            return Session.CreateCriteria<User>()
                 .Add(Restrictions.Eq("State", 3))
                 .List<User>();
        }

        public IList<User> GetUsersByOrderAndRolePaged(int count, int page, out int totalItems, string order, Role role)
        {
            totalItems = Session.CreateCriteria<User>() // vytvoreni kriteria
            .SetProjection(Projections.RowCount()) // spocitani poctu radku
            .UniqueResult<int>(); //vraceni unikatniho vysledku

            IEnumerable<User> users;

            IList<User> usersResult, u;

            int counter = 0;

            switch (order)
            {
                case "upLogin":
                    return Session.CreateCriteria<User>().Add(Restrictions.Eq("Role", role))
                        .AddOrder(Order.Asc("Login"))
                        .SetFirstResult((page - 1) * count)
                        .SetMaxResults(count).List<User>();
                case "downLogin":
                    return Session.CreateCriteria<User>().Add(Restrictions.Eq("Role", role))
                        .AddOrder(Order.Desc("Login"))
                        .SetFirstResult((page - 1) * count)
                        .SetMaxResults(count).List<User>();
                case "upSurname":
                    return Session.CreateCriteria<User>().Add(Restrictions.Eq("Role", role))
                        .AddOrder(Order.Asc("Surname"))
                        .SetFirstResult((page - 1) * count)
                        .SetMaxResults(count).List<User>();
                case "downSurname":
                    return Session.CreateCriteria<User>().Add(Restrictions.Eq("Role", role))
                        .AddOrder(Order.Desc("Surname"))
                        .SetFirstResult((page - 1) * count)
                        .SetMaxResults(count).List<User>();
                case "upDate":
                    return Session.CreateCriteria<User>().Add(Restrictions.Eq("Role", role))
                        .AddOrder(Order.Desc("RegistrationDate"))
                        .SetFirstResult((page - 1) * count)
                        .SetMaxResults(count).List<User>();
                case "downDate":
                    return Session.CreateCriteria<User>().Add(Restrictions.Eq("Role", role))
                        .AddOrder(Order.Asc("RegistrationDate"))
                        .SetFirstResult((page - 1) * count)
                        .SetMaxResults(count).List<User>();
                case "upScore":
                    users = from s in Session.CreateCriteria<User>().Add(Restrictions.Eq("Role", role)).List<User>()
                            select s;

                    usersResult = users.OrderBy(s => s.Score).ToList();

                    u = new List<User>();

                    if (count > usersResult.Count)
                    {
                        while (u.Count < usersResult.Count)
                        {
                            u.Add(usersResult.ElementAt(((page - 1) * usersResult.Count + counter)));
                            counter++;
                        }
                    }
                    else
                    {
                        while (u.Count < count)
                        {
                            u.Add(usersResult.ElementAt(((page - 1) * count + counter)));
                            counter++;
                        }
                    }

                    return u;
                case "downScore":
                    users = from s in Session.CreateCriteria<User>().Add(Restrictions.Eq("Role", role)).List<User>()
                            select s;

                    usersResult = users.OrderByDescending(s => s.Score).ToList();

                    u = new List<User>();

                    if (count > usersResult.Count)
                    {
                        while (u.Count < usersResult.Count)
                        {
                            u.Add(usersResult.ElementAt(((page - 1) * usersResult.Count + counter)));
                            counter++;
                        }
                    }
                    else
                    {
                        while (u.Count < count)
                        {
                            u.Add(usersResult.ElementAt(((page - 1) * count + counter)));
                            counter++;
                        }
                    }

                    return u;
                default:
                    return Session.CreateCriteria<User>().Add(Restrictions.Eq("Role",role))
                        .AddOrder(Order.Asc("Login"))
                        .SetFirstResult((page - 1) * count)
                        .SetMaxResults(count).List<User>();
            }
        }

        public IList<User> GetUsersByOrderPaged(int count, int page, out int totalItems, string order)
        {
            totalItems = Session.CreateCriteria<User>() // vytvoreni kriteria
            .SetProjection(Projections.RowCount()) // spocitani poctu radku
            .UniqueResult<int>(); //vraceni unikatniho vysledku

            IEnumerable<User> users;

            IList<User> usersResult, u;

            int counter = 0;

            switch (order)
            {
                case "upLogin":
                    return Session.CreateCriteria<User>()
                        .AddOrder(Order.Asc("Login"))
                        .SetFirstResult((page - 1) * count)
                        .SetMaxResults(count).List<User>();
                case "downLogin":
                    return Session.CreateCriteria<User>()
                        .AddOrder(Order.Desc("Login"))
                        .SetFirstResult((page - 1) * count)
                        .SetMaxResults(count).List<User>();
                case "upSurname":
                    return Session.CreateCriteria<User>()
                        .AddOrder(Order.Asc("Surname"))
                        .SetFirstResult((page - 1) * count)
                        .SetMaxResults(count).List<User>();
                case "downSurname":
                    return Session.CreateCriteria<User>()
                        .AddOrder(Order.Desc("Surname"))
                        .SetFirstResult((page - 1) * count)
                        .SetMaxResults(count).List<User>();
                case "upDate":
                    return Session.CreateCriteria<User>()
                        .AddOrder(Order.Desc("RegistrationDate"))
                        .SetFirstResult((page - 1) * count)
                        .SetMaxResults(count).List<User>();
                case "downDate":
                    return Session.CreateCriteria<User>()
                        .AddOrder(Order.Asc("RegistrationDate"))
                        .SetFirstResult((page - 1) * count)
                        .SetMaxResults(count).List<User>();
                case "upScore":
                    users = from s in Session.CreateCriteria<User>().List<User>()
                            select s;

                    usersResult = users.OrderBy(s => s.Score).ToList();

                    u = new List<User>();

                    if (count > usersResult.Count)
                    {
                        while (u.Count < usersResult.Count)
                        {
                            u.Add(usersResult.ElementAt(((page - 1) * usersResult.Count + counter)));
                            counter++;
                        }
                    }
                    else
                    {
                        while (u.Count < count)
                        {
                            u.Add(usersResult.ElementAt(((page - 1) * count + counter)));
                            counter++;
                        }
                    }

                    return u;
                case "downScore":
                    users = from s in Session.CreateCriteria<User>().List<User>()
                            select s;

                    usersResult = users.OrderByDescending(s => s.Score).ToList();

                    u = new List<User>();

                    if (count > usersResult.Count)
                    {
                        while (u.Count < usersResult.Count)
                        {
                            u.Add(usersResult.ElementAt(((page - 1) * usersResult.Count + counter)));
                            counter++;
                        }
                    }
                    else
                    {
                        while (u.Count < count)
                        {
                            u.Add(usersResult.ElementAt(((page - 1) * count + counter)));
                            counter++;
                        }
                    }

                    return u;
                default:
                    return Session.CreateCriteria<User>()
                        .AddOrder(Order.Asc("Login"))
                        .SetFirstResult((page - 1) * count)
                        .SetMaxResults(count).List<User>();
            }
        }

        public IList<User> SearchUsers(string phrase)
        {
            return Session.CreateCriteria<User>().Add(Restrictions.Like("Surname", $"%{phrase}%"))
                .List<User>();
        }

        public IList<User> SearchUsersByRole(string phrase, Role role)
        {
            return Session.CreateCriteria<User>().Add(Restrictions.Eq("Role", role)).Add(Restrictions.Like("Surname", $"%{phrase}%"))
                .List<User>();
        }

    }
}
