﻿using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using DataAccess.Interface;
using System.ComponentModel.DataAnnotations.Schema;
using System.Linq;
using DataAccess.Class;
using DataAccess.Dao;

namespace DataAccess.Model
{
    public class User : IEntity
    {
        public virtual int Id { get; set; }

        [Required(ErrorMessage = "Login je vyžadován!")]
        public virtual string Login { get; set; }

        [Required(ErrorMessage = "Heslo je vyžadováno!")]
        public virtual string Password { get; set; }

        [NotMapped]
        [Compare("Password", ErrorMessage = "Heslo se neshoduje!")]
        public string ConfirmPassword { get; set; }

        [NotMapped]
        public string CurrentPassword { get; set; }

        [Required(ErrorMessage = "Jméno je vyžadováno!")]
        public virtual string Name { get; set; }

        [Required(ErrorMessage = "Příjmení je vyžadováno!")]
        public virtual string Surname { get; set; }

        [Required(ErrorMessage = "Pohlaví je vyžadováno!")]
        [CustomRange(new[] { "Žena", "Muž" }, ErrorMessage = "Zadejte pohlaví ze seznamu!")]
        public virtual string Gender { get; set; }

        [EmailAddress(ErrorMessage = "Zadejte Email ve správném tvaru!")]
        public virtual string Email { get; set; }

        [NotMapped]
        private float _score;

        [NotMapped]
        public virtual float Score
        {
            get
            {
                IList<Test> tests = new TestDao().GetByUser(new UserDao().GetById(Id));

                float grade = tests.Sum(test => test.Grade);

                if (tests.Count != 0)
                {
                    _score = grade / tests.Count;
                }

                return _score;
            }
            set { _score = value; }
        }

        [NotMapped]
        public virtual bool Worthy
        {
            get
            {
                IList<Chapter> chapters = new ChapterDao().GetAll();

                int counter = 0;

                float testsGrade = 0;

                foreach (IList<Test> tests in chapters.Select(chapter => new TestDao().GetByChapterAndUser(new UserDao().GetById(Id), chapter)))
                {
                    testsGrade = tests.Aggregate(testsGrade, (current, test) => current + test.Grade);

                    float testResult = testsGrade / tests.Count;

                    if (tests.Count >= 5 && testResult <= 2.5)
                    {
                        counter++;
                    }
                }

                return counter == chapters.Count;
            }
        }

        public virtual DateTime RegistrationDate { get; set; }

        public virtual int State { get; set; }

        public virtual Role Role { get; set; }
    }
}
