﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web.Mvc;
using DataAccess.Dao;
using DataAccess.Model;

namespace PRIPO_v02.Areas.Admin.Controllers
{
    [Authorize(Roles = "admin")]
    public class ArticleController : Controller
    {
        // GET: Admin/Article
        public ActionResult Index(int? page)
        {
            const int itemsOnPage = 10;
            int pg = page ?? 1;
            int totalItems;

            ViewBag.CurrentPage = pg;
            ViewBag.ItemsOnPage = itemsOnPage;

            IList<Article> articles = new ArticleDao().GetArticlesPaged(itemsOnPage, pg, out totalItems);

            if (Request.IsAjaxRequest())
            {
                return PartialView(articles);
            }

            return View(articles);
        }

        [ChildActionOnly]
        public ActionResult IndexArticles(int? page)
        {
            const int itemsOnPage = 5;
            int pg = page ?? 1;
            int totalItems;

            ViewBag.CurrentPage = pg;
            ViewBag.ItemsOnPage = itemsOnPage;

            IList<Article> articles = new ArticleDao().GetArticlesPaged(itemsOnPage, pg, out totalItems);

            if (Request.IsAjaxRequest())
            {
                return PartialView(articles);
            }

            return View(articles);
        }

        public JsonResult SearchRecords(string query)
        {

            IList<Article> articles = new ArticleDao().SearchArticles(query);

            IList<string> responses = (from Article article in articles select article.Title).ToList();
            IList<string> responses1 = new List<string>();

            foreach (string response in responses.Where(response => !responses1.Contains(response)))
            {
                responses1.Add(response);
            }

            return Json(responses1, JsonRequestBehavior.AllowGet);

        }

        public ActionResult Search(string phrase)
        {

            IList<Article> articles = new ArticleDao().SearchArticles(phrase);

            if (Request.IsAjaxRequest())
            {
                return PartialView("Index", articles);
            }

            return View("Index", articles);
        }

        public ActionResult Detail(int id)
        {
            Article article = new ArticleDao().GetById(id);

            if (Request.IsAjaxRequest())
            {
                return PartialView(article);
            }

            return View(article);
        }

        public ActionResult Create()
        {
            return View();
        }

        [HttpPost]
        public ActionResult Add(Article article)
        {
            if (ModelState.IsValid)
            {
                ArticleDao articleDao = new ArticleDao();

                article.User = new UserDao().GetByLogin(User.Identity.Name);

                article.AddedDate = DateTime.Now;

                articleDao.Create(article);

                TempData["message-success"] = "Článek byl úspěšně přidán";
            }
            else
            {
                return View("Create", article);
            }

            return RedirectToAction("Index", "Article");
        }

        public ActionResult Edit(int id)
        {
            Article article = new ArticleDao().GetById(id);

            return View(article);
        }

        [HttpPost]
        public ActionResult Update(Article article)
        {
            if (ModelState.IsValid)
            {

                ArticleDao articleDao = new ArticleDao();

                User user = new UserDao().GetByLogin(User.Identity.Name);

                article.User = user;

                article.AddedDate = DateTime.Now;

                articleDao.Update(article);

                TempData["message-success"] = "Článek " + article.Title + " byl úspěšně upraven";
            }
            else
            {
                return View("Create", article);
            }

            return RedirectToAction("Index", "Article");
        }

        public ActionResult Delete(int id)
        {
            ArticleDao articleDao = new ArticleDao();
            Article article = articleDao.GetById(id);

            articleDao.Delete(article);
            TempData["message-success"] = "Článek " + article.Title + " byl úspěšně smazán";


            return RedirectToAction("Index", "Article");
        }
    }
}