﻿using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using DataAccess.Dao;
using DataAccess.Model;

namespace PRIPO_v02.Areas.Admin.Controllers
{
    [Authorize(Roles = "admin")]
    public class ChapterController : Controller
    {
        // GET: Admin/Chapter
        public ActionResult Index(int? page)
        {
            const int itemsOnPage = 8;
            int pg = page ?? 1;
            int totalItems;

            ViewBag.CurrentPage = pg;
            ViewBag.ItemsOnPage = itemsOnPage;

            IList<Chapter> chapters = new ChapterDao().GetChaptersPaged(itemsOnPage, pg, out totalItems);

            if (Request.IsAjaxRequest())
            {
                return PartialView(chapters);
            }

            return View(chapters);
        }

        public JsonResult SearchRecords(string query)
        {

            IList<Chapter> chapters = new ChapterDao().SearchChapters(query);

            IList<string> responses = (from Chapter chapter in chapters select chapter.Name).ToList();

            IList<string> responses1 = new List<string>();

            foreach (string response in responses.Where(response => !responses1.Contains(response)))
            {
                responses1.Add(response);
            }

            return Json(responses1, JsonRequestBehavior.AllowGet);
        }

        public ActionResult Search(string phrase)
        {

            IList<Chapter> chapters = new ChapterDao().SearchChapters(phrase);

            if (Request.IsAjaxRequest())
            {
                return PartialView("Index", chapters);
            }

            return View("Index", chapters);
        }

        public ActionResult Detail(int id)
        {
            Chapter chapter = new ChapterDao().GetById(id);

            if (Request.IsAjaxRequest())
            {
                return PartialView(chapter);
            }

            return View(chapter);
        }

        public ActionResult Create()
        {
            return View();
        }

        [HttpPost]
        public ActionResult Add(Chapter chapter, HttpPostedFileBase presentation)
        {
            if (ModelState.IsValid)
            {
                ChapterDao chapterDao = new ChapterDao();

                if (presentation != null)
                {
                    switch (presentation.ContentType)
                    {
                        case "application/pdf":

                            presentation.SaveAs(Server.MapPath("~/Uploads/Presentations/" + chapter.Name + ".pdf"));

                            chapter.Presentation = chapter.Name + ".pdf";
                            break;
                        case "application/vnd.openxmlformats-officedocument.presentationml.presentation":

                            presentation.SaveAs(Server.MapPath("~/Uploads/Presentations/" + chapter.Name + ".pptx"));

                            chapter.Presentation = chapter.Name + ".pptx";
                            break;
                        case "application/vnd.ms-powerpoint":
                            presentation.SaveAs(Server.MapPath("~/Uploads/Presentations/" + chapter.Name + ".ppt"));

                            chapter.Presentation = chapter.Name + ".ppt";
                            break;
                        default:
                            TempData["message-error"] = "Pokoušíte se nahrát neznámý soubor pro prezentaci";
                            return View("Create", chapter);
                    }
                }
                else
                {
                    chapter.Presentation = "Prezentace nebyla přiložena";
                }
                chapterDao.Create(chapter);

                TempData["message-success"] = "Kapitola " + chapter.Name + " byla úspěšně vytvořena";

            }
            else
            {
                return View("Create", chapter);
            }

            return RedirectToAction("Index", "Chapter");
        }

        public ActionResult Edit(int id)
        {
            Chapter chapter = new ChapterDao().GetById(id);

            return View(chapter);
        }

        [HttpPost]
        public ActionResult Update(Chapter chapter, HttpPostedFileBase presentation)
        {
            if (ModelState.IsValid)
            {
                ChapterDao chapterDao = new ChapterDao();

                if ((Request.Form["presentationControl"] != "Prezentace nebyla přiložena" || string.IsNullOrEmpty(Request.Form["presentationControl"])) && presentation != null)
                {
                    switch (presentation.ContentType)
                    {
                        case "application/pdf":
                            if (System.IO.File.Exists(Server.MapPath("~/Uploads/Presentations/" + chapter.Presentation)))
                            {
                                System.IO.File.Delete(Server.MapPath("~/Uploads/Presentations/" + chapter.Presentation));
                            }

                            presentation.SaveAs(Server.MapPath("~/Uploads/Presentations/" + chapter.Name + ".pdf"));

                            chapter.Presentation = chapter.Name + ".pdf";
                            break;
                        case "application/vnd.openxmlformats-officedocument.presentationml.presentation":
                            if (System.IO.File.Exists(Server.MapPath("~/Uploads/Presentations/" + chapter.Presentation)))
                            {
                                System.IO.File.Delete(Server.MapPath("~/Uploads/Presentations/" + chapter.Presentation));
                            }

                            presentation.SaveAs(Server.MapPath("~/Uploads/Presentations/" + chapter.Name + ".pptx"));

                            chapter.Presentation = chapter.Name + ".pptx";
                            break;
                        case "application/vnd.ms-powerpoint":
                            if (System.IO.File.Exists(Server.MapPath("~/Uploads/Presentations/" + chapter.Presentation)))
                            {
                                System.IO.File.Delete(Server.MapPath("~/Uploads/Presentations/" + chapter.Presentation));
                            }

                            presentation.SaveAs(Server.MapPath("~/Uploads/Presentations/" + chapter.Name + ".ppt"));

                            chapter.Presentation = chapter.Name + ".ppt";
                            break;
                        default:
                            TempData["message-error"] = "Pokoušíte se nahrát neznámý soubor pro prezentaci";
                            return View("Edit", chapter);
                    }
                }
                else
                {
                    if (Request.Form["presentationControl"] != "Nezměněno")
                    {
                        if (System.IO.File.Exists(Server.MapPath("~/Uploads/Presentations/" + chapter.Presentation)))
                        {
                            System.IO.File.Delete(Server.MapPath("~/Uploads/Presentations/" + chapter.Presentation));
                        }
                        chapter.Presentation = "Prezentace nebyla přiložena";
                    }
                }
                chapterDao.Update(chapter);

                TempData["message-success"] = "Kapitola " + chapter.Name + " byla úspěšně upravena";
            }
            else
            {
                return View("Edit", chapter);
            }
            return RedirectToAction("Index", "Chapter");
        }

        public FileResult Download(int id)
        {
            Chapter chapter = new ChapterDao().GetById(id);

            var filepath = Path.Combine(Server.MapPath("~/Uploads/Presentations/"), chapter.Presentation);
            return File(filepath, MimeMapping.GetMimeMapping(filepath), chapter.Presentation);
        }

        public ActionResult Delete(int id)
        {
            ChapterDao chapterDao = new ChapterDao();
            Chapter chapter = chapterDao.GetById(id);

            QuestionDao questionDao = new QuestionDao();

            IList<Question> questions = questionDao.GetByChapter(chapter);

            if (System.IO.File.Exists(Server.MapPath("~/Uploads/Presentations/" + chapter.Presentation)))
            {
                System.IO.File.Delete(Server.MapPath("~/Uploads/Presentations/" + chapter.Presentation));
            }

            if (questions.Count == 0)
            {
                chapterDao.Delete(chapter);

                TempData["message-success"] = "Kapitola " + chapter.Name + " byla úspěšně smazána";

                return RedirectToAction("Index", "Chapter");

            }

            TestDao testDao = new TestDao();
            IList<Test> tests = testDao.GetByChapter(chapter);

            AnswerDao answerDao = new AnswerDao();

            if (tests.Count != 0)
            {
                QuestionsOfTestDao questionsOfTestDao = new QuestionsOfTestDao();

                foreach (Test test in tests)
                {
                    IList<QuestionsOfTest> questionsOfTests = questionsOfTestDao.GetByTest(test);

                    if (questionsOfTests.Count != 0)
                    {
                        foreach (QuestionsOfTest questionsOfTest in questionsOfTests)
                        {

                            IList<Answer> answers = answerDao.GetByQuestionsOfTestsAndType(questionsOfTest, 2);

                            foreach (Answer answer in answers)
                            {
                                answerDao.Delete(answer);
                            }

                            questionsOfTestDao.Delete(questionsOfTest);

                        }

                        test.Questions = null;

                        testDao.Delete(test);
                    }
                    else
                    {
                        testDao.Delete(test);
                    }
                }
            }

            foreach (Question question in questions)
            {
                IList<Answer> answers = answerDao.GetByQuestionAndType(question, 1);

                foreach (Answer answer in answers)
                {
                    answerDao.Delete(answer);
                }

                questionDao.Delete(question);
            }

            chapterDao.Delete(chapter);

            TempData["message-success"] = "Kapitola " + chapter.Name + " byla úspěšně smazána";

            return RedirectToAction("Index", "Chapter");
        }
    }
}