﻿using System.Collections.Generic;
using System.Linq;
using System.Web.Mvc;
using DataAccess.Dao;
using DataAccess.Model;

namespace PRIPO_v02.Areas.Admin.Controllers
{
    [Authorize(Roles = "admin")]
    public class QuestionController : Controller
    {
        // GET: Admin/Question
        public ActionResult Index(int? page)
        {
            const int itemsOnPage = 10;
            int pg = page ?? 1;
            int totalItems;

            ViewBag.CurrentPage = pg;
            ViewBag.ItemsOnPage = itemsOnPage;

            IList<Question> questions = new QuestionDao().GetQuestionsPaged(itemsOnPage, pg, out totalItems);

            if (Request.IsAjaxRequest())
            {
                return PartialView(questions);
            }

            return View(questions);
        }

        public JsonResult SearchRecords(string query)
        {

            IList<Question> questions = new QuestionDao().SearchQuestions(query);

            IList<string> responses = (from Question question in questions select question.Chapter.Name).ToList();

            IList<string> responses1 = new List<string>();

            foreach (string response in responses.Where(response => !responses1.Contains(response)))
            {
                responses1.Add(response);
            }

            return Json(responses1, JsonRequestBehavior.AllowGet);
        }

        public ActionResult Search(string phrase)
        {

            IList<Question> questions = new QuestionDao().SearchQuestions(phrase);

            if (Request.IsAjaxRequest())
            {
                return PartialView("Index", questions);
            }

            return View("Index", questions);
        }

        public ActionResult Create()
        {
            ViewBag.Chapters = new ChapterDao().GetAll();

            return View();
        }

        [HttpPost]
        public ActionResult Add(Question question, int chapterId)
        {
            if (ModelState.IsValid)
            {

                QuestionDao questionDao = new QuestionDao();

                AnswerDao answerDao = new AnswerDao();

                Chapter chapter = new ChapterDao().GetById(chapterId);

                string answered = Request.Form["checkbox"];

                string[] answers = answered.Split(',');

                question.Chapter = chapter;
                questionDao.Create(question);

                foreach (string answer1 in answers)
                {
                    if (answer1 == "false" || answer1 == string.Empty) continue;
                    Answer answer = new Answer
                    {
                        Type = 1,
                        Question = question
                    };

                    switch (answer1)
                    {
                        case "A":
                            answer.Text = question.A;
                            break;
                        case "B":
                            answer.Text = question.B;
                            break;
                        case "C":
                            answer.Text = question.C;
                            break;
                        case "D":
                            answer.Text = question.D;
                            break;
                        default:
                            answer.Text = answer1;
                            break;
                    }

                    answerDao.Create(answer);
                }

                TempData["message-success"] = "Otázka z Kapitoly " + question.Chapter.Name + " byla úspěšně vytvořena";

            }
            else
            {
                ViewBag.Chapters = new ChapterDao().GetAll();
                return View("Create", question);
            }

            return RedirectToAction("Index", "Question", question);
        }

        public ActionResult Edit(int id)
        {
            Question question = new QuestionDao().GetById(id);

            AnswerDao answerDao = new AnswerDao();

            IList<Answer> answers = answerDao.GetByQuestion(question);

            foreach (Answer answer in answers)
            {

                if (answer.Text == question.A)
                {
                    ViewBag.Answer += "A,";
                }
                else if (answer.Text == question.B)
                {
                    ViewBag.Answer += "B,";
                }
                else if (answer.Text == question.B)
                {
                    ViewBag.Answer += "C,";
                }
                else if (answer.Text == question.B)
                {
                    ViewBag.Answer += "D,";
                }
                else
                {
                    ViewBag.Answer += answer.Text + ",";
                }
            }

            ViewBag.Chapters = new ChapterDao().GetAll();

            return View(question);
        }

        [HttpPost]
        public ActionResult Update(Question question, int chapterId)
        {
            if (ModelState.IsValid)
            {

                QuestionDao questionDao = new QuestionDao();

                AnswerDao answerDao = new AnswerDao();

                Chapter chapter = new ChapterDao().GetById(chapterId);

                IList<Answer> rightAnswers = answerDao.GetByQuestionAndType(question, 1);

                foreach (Answer answer in rightAnswers)
                {
                    answerDao.Delete(answer);
                }

                string answered = Request.Form["checkbox"];

                string[] answers = answered.Split(',');

                question.Chapter = chapter;
                questionDao.Update(question);

                foreach (string answer1 in answers)
                {
                    if (answer1 == "false" || answer1 == string.Empty) continue;
                    Answer answer = new Answer
                    {
                        Type = 1,
                        Question = question
                    };

                    switch (answer1)
                    {
                        case "A":
                            answer.Text = question.A;
                            break;
                        case "B":
                            answer.Text = question.B;
                            break;
                        case "C":
                            answer.Text = question.C;
                            break;
                        case "D":
                            answer.Text = question.D;
                            break;
                        default:
                            answer.Text = answer1;
                            break;
                    }

                    answerDao.Create(answer);

                }

                TempData["message-success"] = "Otázka z Kapitoly " + question.Chapter.Name + " byla úspěšně upravena";

            }
            else
            {
                ViewBag.Chapters = new ChapterDao().GetAll();
                return View("Edit", question);
            }

            return RedirectToAction("Index", "Question", question);
        }

        public ActionResult Delete(int id)
        {
            QuestionDao questionDao = new QuestionDao();
            Question question = questionDao.GetById(id);

            IList<QuestionsOfTest> questionsOfTests = new QuestionsOfTestDao().GetByQuestion(question);

            if (questionsOfTests.Count == 0)
            {
                AnswerDao answerDao = new AnswerDao();

                foreach (Answer answer in answerDao.GetByQuestion(question))
                {
                    answerDao.Delete(answer);
                }

                questionDao.Delete(question);
                TempData["message-success"] = "Otázka z Kapitoly " + question.Chapter.Name + " byla úspěšně smazána";
            }
            else
            {
                TempData["message-error"] = "Otázka z Kapitoly " + question.Chapter.Name + " je obsažena v testu";
            }

            return RedirectToAction("Index", "Question");
        }

        public ActionResult Answer(int id)
        {

            QuestionsOfTest questionsOfTest = new QuestionsOfTestDao().GetById(id);

            AnswerDao answerDao = new AnswerDao();

            ViewBag.RightAnswers = answerDao.GetByQuestionAndType(questionsOfTest.Question, 1);
            ViewBag.Answers = answerDao.GetByQuestionsOfTestsAndType(questionsOfTest, 2);

            if (Request.IsAjaxRequest())
            {
                return PartialView(questionsOfTest);
            }
            return View(questionsOfTest);
        }

        public ActionResult Detail(int id)
        {

            Question question = new QuestionDao().GetById(id);

            ViewBag.Answers = new AnswerDao().GetByQuestionAndType(question, 1);

            if (Request.IsAjaxRequest())
            {
                return PartialView(question);
            }
            return View(question);
        }
    }

}