﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web.Mvc;
using System.Web.Security;
using DataAccess.Dao;
using DataAccess.Model;
using PRIPO_v02.Class;

namespace PRIPO_v02.Areas.Admin.Controllers
{
    [Authorize(Roles = "admin")]
    public class UserController : Controller
    {
        public ActionResult Index(int? page, string order)
        {
            const int itemsOnPage = 10;
            int pg = page ?? 1;
            int totalItems;

            UserDao userDao = new UserDao();

            TestHelper.ClearTests(null);

            IList<User> deactivatedUsers = userDao.GetAllDeactivated();

            foreach (User user in deactivatedUsers)
            {
                TimeSpan difference = DateTime.Now - user.RegistrationDate;
                switch (user.State)
                {
                    case 2:
                        if (difference.TotalDays > 360)
                        {
                            Delete(user.Id);
                            TempData["message-success"] = "Uživatelské účty byly úspěšně vyčištěny";
                        }
                        break;
                    default:
                        if (difference.TotalDays > 30)
                        {
                            Delete(user.Id);
                            TempData["message-success"] = "Uživatelské účty byly úspěšně vyčištěny";
                        }
                        break;
                }


            }

            IList<User> users = new UserDao().GetUsersByOrderPaged(itemsOnPage, pg, out totalItems, order);

            ViewBag.CurrentPage = pg;
            ViewBag.ItemsOnPage = itemsOnPage;
            ViewBag.Roles = new RoleDao().GetAll();
            ViewBag.Order = order;

            if (Request.IsAjaxRequest())
            {
                return PartialView("Index", users);
            }

            return View("Index", users);
        }

        public JsonResult SearchRecords(string query)
        {

            IList<User> users = new UserDao().SearchUsers(query);

            IList<string> responses = (from User user in users select user.Surname).ToList();

            ViewBag.Roles = new RoleDao().GetAll();

            IList<string> responses1 = new List<string>();

            foreach (string response in responses.Where(response => !responses1.Contains(response)))
            {
                responses1.Add(response);
            }

            return Json(responses1, JsonRequestBehavior.AllowGet);
        }

        public ActionResult Search(string phrase)
        {

            IList<User> users = new UserDao().SearchUsers(phrase);

            ViewBag.Roles = new RoleDao().GetAll();

            if (Request.IsAjaxRequest())
            {
                return PartialView("Index", users);
            }

            return View("Index", users);
        }

        public ActionResult Detail(int id)
        {
            User user = new UserDao().GetById(id);

            if (Request.IsAjaxRequest())
            {
                return PartialView(user);
            }

            return View(user);
        }

        public ActionResult Edit(int id)
        {

            User user = new UserDao().GetById(id);
            ViewBag.Roles = new RoleDao().GetAll();

            return View("Edit", user);
        }

        public ActionResult EditPassword(int id)
        {

            User user = new UserDao().GetById(id);

            return View("EditPassword", user);
        }

        [HttpPost]
        public ActionResult Update(User user, int? roleId)
        {
            if (ModelState.IsValid)
            {

                UserDao userDao = new UserDao();

                User userOld = userDao.GetById(user.Id);

                if (roleId.HasValue)
                {
                    user.Role = new RoleDao().GetById((int)roleId);
                }

                string oldlogin = userOld.Login;

                if (userDao.GetByLogin(user.Login) != null && user.Login != oldlogin)
                {
                    TempData["message-error"] = "Uživatelské jméno " + user.Login + " již je v databázi";

                    return View("Edit", user);
                }
                userDao.Merge(user);
                TempData["message-success"] = "Účet uživatele " + user.Name + " " + user.Surname + " byl úspěšně upraven";
            }
            else
            {
                ViewBag.Roles = new RoleDao().GetAll();
                return View("Edit", user);

            }
            return RedirectToAction("Index", "User");
        }

        [HttpPost]
        public ActionResult UpdatePassword(User user)
        {
            if (ModelState.IsValid)
            {

                UserDao userDao = new UserDao();

                if (userDao.GetByLogin(User.Identity.Name) == user)
                {
                    User userOld = userDao.GetById(user.Id);

                    user.Role = new RoleDao().GetById(userOld.Role.Id);

                    if (PasswordHash.ValidatePassword(user.CurrentPassword, userOld.Password))
                    {
                        user.Password = PasswordHash.CreateHash(user.ConfirmPassword);
                        userDao.Merge(user);
                        TempData["message-success"] = "Heslo bylo úspěšně změněno";
                    }
                    else
                    {
                        TempData["message-error"] = "Heslo nesouhlasí";
                        return RedirectToAction("EditPassword", "User", new { id = user.Id });
                    }
                }
                else
                {
                    User userOld = userDao.GetById(user.Id);

                    user.Role = new RoleDao().GetById(userOld.Role.Id);

                    user.Password = PasswordHash.CreateHash(user.ConfirmPassword);
                    userDao.Merge(user);
                    TempData["message-success"] = "Heslo bylo úspěšně změněno";
                }
            }
            else
            {
                return View("EditPassword", user);
            }

            return View("Detail", user);

        }

        [HttpPost]
        public ActionResult UpdateRole(int userId, int roleId)
        {
            if (!ModelState.IsValid) return RedirectToAction("Index");

            UserDao userDao = new UserDao();

            User user = userDao.GetById(userId);

            user.Role = new RoleDao().GetById(roleId);

            userDao.Merge(user);
            TempData["message-success"] = "Role uživatele " + user.Name + " " + user.Surname + " byla úspěšně změněna";
            return RedirectToAction("Index");

        }

        [HttpPost]
        public ActionResult ActivateAcount(int userId)
        {
            if (!ModelState.IsValid) return RedirectToAction("Index");

            UserDao userDao = new UserDao();

            User user = userDao.GetById(userId);

            user.State = 1;

            userDao.Merge(user);
            TempData["message-success"] = "Uživatelský účet " + user.Name + " " + user.Surname + " byl úspěšně aktivován";

            return RedirectToAction("Index");

        }

        [HttpPost]
        public ActionResult DeactivateAcount(int userId)
        {
            if (!ModelState.IsValid) return RedirectToAction("Index");

            UserDao userDao = new UserDao();

            User user = userDao.GetById(userId);

            user.State = 2;

            userDao.Merge(user);
            TempData["message-success"] = "Uživatelský účet " + user.Name + " " + user.Surname + " byl úspěšně deaktivován";

            return RedirectToAction("Index");

        }

        [HttpPost]
        public ActionResult ActivateAllRegisteredAcounts()
        {
            if (!ModelState.IsValid) return RedirectToAction("Index");

            UserDao userDao = new UserDao();

            IList<User> users = userDao.GetAllRegistered();

            if (users.Count == 0)
            {
                TempData["message-error"] = "Nebyl nalezen žádný neaktivovaný účet";

                return RedirectToAction("Index");
            }

            foreach (User user in users)
            {
                user.State = 1;
                userDao.Merge(user);
            }

            TempData["message-success"] = "Všichni registrovaní uživatelé byli úspěšně aktivováni";

            return RedirectToAction("Index");

        }

        public ActionResult Delete(int id)
        {
            UserDao userDao = new UserDao();
            User user = userDao.GetById(id);

            ArticleDao articleDao = new ArticleDao();
            IList<Article> articles = articleDao.GetByUser(user);

            if (articles.Count != 0)
            {
                foreach (Article article in articles)
                {
                    articleDao.Delete(article);
                }
            }

            TestDao testDao = new TestDao();
            IList<Test> tests = testDao.GetByUser(user);

            if (tests.Count != 0)
            {
                AnswerDao answerDao = new AnswerDao();
                QuestionsOfTestDao questionsOfTestDao = new QuestionsOfTestDao();

                foreach (Test test in tests)
                {
                    IList<QuestionsOfTest> questionsOfTests = questionsOfTestDao.GetByTest(test);

                    if (questionsOfTests.Count != 0)
                    {
                        foreach (QuestionsOfTest questionsOfTest in questionsOfTests)
                        {

                            IList<Answer> answers = answerDao.GetByQuestionsOfTestsAndType(questionsOfTest, 2);

                            foreach (Answer answer in answers)
                            {
                                answerDao.Delete(answer);
                            }

                            questionsOfTestDao.Delete(questionsOfTest);

                        }

                        test.Questions = null;

                        testDao.Delete(test);
                    }
                    else
                    {
                        testDao.Delete(test);
                    }
                }
            }

            userDao.Delete(user);

            if (User.Identity.IsAuthenticated)
            {
                TempData["message-success"] = "Účet byl úspěšně smazán";
                return RedirectToAction("Index", "User");
            }
            FormsAuthentication.SignOut();
            Session.Clear();

            TempData["message-success"] = "Váš účet byl úspěšně smazán";
            return RedirectToAction("Index", "Home", new { area = "" });
        }

        public ActionResult Create()
        {

            ViewBag.Roles = new RoleDao().GetAll();

            return View();
        }

        [HttpPost]
        public ActionResult Add(User user, int roleId)
        {
            if (ModelState.IsValid)
            {
                if (user.Password != Request.Form["ConfirmPassword"]) return RedirectToAction("Index", "User");
                UserDao userDao = new UserDao();
                Role role = new RoleDao().GetById(roleId);

                user.Role = role;

                user.RegistrationDate = DateTime.Now;

                user.State = 1;

                if (userDao.GetByLogin(user.Login) != null)
                {
                    TempData["message-error"] = "uživatelské jméno " + user.Login + " již je v databázi";

                    return View("Create");
                }
                user.Password = PasswordHash.CreateHash(Request.Form["ConfirmPassword"]);
                userDao.Create(user);
                TempData["message-success"] = "Účet byl úspěšně vytvořen";
            }
            else
            {
                ViewBag.Roles = new RoleDao().GetAll();
                return View("Create", user);

            }

            return RedirectToAction("Index", "User");
        }
    }
}
