﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web.Mvc;
using DataAccess.Dao;
using DataAccess.Model;
using PRIPO_v02.Class;

namespace PRIPO_v02.Areas.Student.Controllers
{
    [Authorize]
    public class TestController : Controller
    {
        // GET: Student/Test
        public ActionResult Index(int id)
        {

            Test test = new TestDao().GetByIdAndUser(id, new UserDao().GetByLogin(User.Identity.Name));

            if (test == null)
            {
                TempData["message-error"] = "Nejspíš se pokoušíte vstoupit do oblasti, pro kterou nemáte dostatečné oprávnění.";

                return RedirectToAction("Index", "Home");
            }

            if (Math.Abs(test.Grade) > 0)
            {
                TempData["message-error"] = "Tento test již byl oznámkován";
                return RedirectToAction("IndexAll", "Test");
            }
            IList<QuestionsOfTest> questionsOfTest = new QuestionsOfTestDao().GetByTest(test);

            ViewBag.Chapter = test.Chapter.Name;
            ViewBag.Questions = questionsOfTest;

            if (Request.IsAjaxRequest())
            {
                return PartialView(test);
            }

            return View(test);
        }

        public ActionResult IndexAll(int? page, string order)
        {
            const int itemsOnPage = 10;
            int pg = page ?? 1;
            int totalItems;

            User user = new UserDao().GetByLogin(User.Identity.Name);

            TestDao testDao = new TestDao();

            TestHelper.ClearTests(user.Id);

            IList<Test> tests = testDao.GetTestsByUserAndOrderPaged(itemsOnPage, pg, out totalItems, order, user);

            ViewBag.CurrentPage = pg;
            ViewBag.ItemsOnPage = itemsOnPage;
            ViewBag.Order = order;

            if (Request.IsAjaxRequest())
            {
                return PartialView(tests);
            }
            return View(tests);
        }

        public JsonResult SearchRecords(string query)
        {

            IList<Test> tests = new TestDao().SearchTestsByUser(query, new UserDao().GetByLogin(User.Identity.Name));

            IList<string> responses = (from Test test in tests select test.Chapter.Name).ToList();

            IList<string> responses1 = new List<string>();

            foreach (string response in responses.Where(response => !responses1.Contains(response)))
            {
                responses1.Add(response);
            }

            return Json(responses1, JsonRequestBehavior.AllowGet);
        }

        public ActionResult Search(string phrase)
        {

            IList<Test> tests = new TestDao().SearchTestsByUser(phrase, new UserDao().GetByLogin(User.Identity.Name));

            if (Request.IsAjaxRequest())
            {
                return PartialView("IndexAll", tests);
            }

            return View("IndexAll", tests);
        }

        public ActionResult Evaluated(int id)
        {

            Test test = new TestDao().GetByIdAndUser(id, new UserDao().GetByLogin(User.Identity.Name));

            if (test == null)
            {
                TempData["message-error"] = "Pokoušíte se vstoupit do oblasti, pro kterou nemáte dostatečné oprávnění.";

                return RedirectToAction("Index", "Home");
            }

            IList<QuestionsOfTest> questionsOfTests = new QuestionsOfTestDao().GetByTest(test);

            IList<Answer> answers = new List<Answer>();

            foreach (QuestionsOfTest questionsOfTest in questionsOfTests)
            {
                foreach (Answer answer in new AnswerDao().GetByQuestionAndType(questionsOfTest.Question, 2))
                {
                    answers.Add(answer);
                }
            }

            ViewBag.Answers = answers;

            return View(test);
        }

        public ActionResult Generate(int chapterId)
        {

            User user = new UserDao().GetByLogin(User.Identity.Name);

            Chapter chapter = new ChapterDao().GetById(chapterId);

            QuestionDao questionDao = new QuestionDao();

            IList<Question> questions;

            QuestionsOfTestDao questionsOfTestDao = new QuestionsOfTestDao();

            Random r = new Random();
            Test test = new Test
            {
                ExecutionDate = DateTime.Now,
                User = user
            };

            TestDao testDao = new TestDao();

            IList<int> questionsIds;

            int counter = 0;

            switch (chapterId)
            {
                case 100000:

                    questions = questionDao.GetAll();

                    if (questions.Count < 50)
                    {
                        TempData["message-error"] = "Omlouváme se, ale pro vytvoření testu nebyl zadán dostatečný počet otázek";

                        return RedirectToAction("Index", "Chapter");
                    }

                    test.Chapter = null;

                    questionsIds = questions.Select(question => question.Id).ToList();

                    testDao.Create(test);

                    while (counter < 50)
                    {
                        counter++;
                        int rInt = r.Next(0, questionsIds.Count); //for ints

                        int questionId = questionsIds.ElementAt(rInt);

                        questionsIds.RemoveAt(rInt);

                        Question question = questionDao.GetById(questionId);

                        QuestionsOfTest questionsOfTest = new QuestionsOfTest
                        {
                            Question = question,
                            Number = counter,
                            Test = test
                        };

                        questionsOfTestDao.Create(questionsOfTest);
                    }

                    return RedirectToAction("Index", new { id = test.Id });

                default:
                    questions = questionDao.GetByChapter(chapter);

                    if (questions.Count < chapter.NumberOfQuestions)
                    {
                        TempData["message-error"] = "Omlouváme se, ale pro vytvoření testu nebyl zadán dostatečný počet otázek";

                        return RedirectToAction("Detail", "Chapter", new { id = chapter.Id });
                    }

                    questionsIds = questions.Select(question => question.Id).ToList();

                    test.Chapter = chapter;

                    testDao.Create(test);

                    while (counter < chapter.NumberOfQuestions)
                    {
                        counter++;
                        int rInt = r.Next(0, questionsIds.Count); //for ints

                        int questionId = questionsIds.ElementAt(rInt);

                        questionsIds.RemoveAt(rInt);

                        Question question = questionDao.GetById(questionId);

                        QuestionsOfTest questionsOfTest = new QuestionsOfTest
                        {
                            Question = question,
                            Number = counter,
                            Test = test
                        };

                        questionsOfTestDao.Create(questionsOfTest);
                    }

                    return RedirectToAction("Index", new { id = test.Id });
            }
        }

        public ActionResult Evaluate(int id)
        {
            TestDao testDao = new TestDao();
            Test test = testDao.GetById(id);

            if (Math.Abs(test.Grade) > 0)
            {
                return RedirectToAction("Evaluated", "Test", new { id = test.Id });
            }

            AnswerDao answerDao = new AnswerDao();

            QuestionsOfTestDao questionsOfTestDao = new QuestionsOfTestDao();

            IList<QuestionsOfTest> questionsOfTests = questionsOfTestDao.GetByTest(test);

            float grade = 0;

            IList<Answer> answers = new List<Answer>();

            foreach (QuestionsOfTest questionsOfTest in questionsOfTests)
            {
                string answered = Request.Form[questionsOfTest.Number.ToString()];

                answers.Clear();

                foreach (string word in answered.Split(','))
                {
                    if (word == "false") continue;

                    Answer answer = new Answer
                    {
                        Text = word,
                        Type = 2,
                        QuestionsOfTest = questionsOfTest
                    };

                    answerDao.Create(answer);

                    answers.Add(answer);
                }

                questionsOfTest.State = TestHelper.Compare(answerDao.GetByQuestionAndType(questionsOfTest.Question, 1), answers, questionsOfTest.Question.Type);

                questionsOfTestDao.Update(questionsOfTest);

                if (questionsOfTest.State)
                {
                    grade += 1;
                }
            }

            float result = grade / questionsOfTests.Count;

            test.Grade = TestHelper.GetGrade(result);

            testDao.Update(test);

            return RedirectToAction("Evaluated", new { id = test.Id });
        }
    }
}
