﻿using System.Collections.Generic;
using System.Linq;
using System.Web.Mvc;
using DataAccess.Dao;
using DataAccess.Model;
using PRIPO_v02.Class;

namespace PRIPO_v02.Areas.Teacher.Controllers
{
    [Authorize(Roles = "admin, teacher")]
    public class TestController : Controller
    {
        // GET: Teacher/Test
        public ActionResult Index(int? page, int id, string order)
        {
            const int itemsOnPage = 10;
            int pg = page ?? 1;
            int totalItems;

            User user = new UserDao().GetById(id);

            TestHelper.ClearTests(user.Id);

            IList<Test> tests = new TestDao().GetTestsByUserAndOrderPaged(itemsOnPage, pg, out totalItems, order, user);

            ViewBag.CurrentPage = pg;
            ViewBag.ItemsOnPage = itemsOnPage;
            ViewBag.Order = order;

            ViewBag.User = user.Name + " " + user.Surname;
            if (Request.IsAjaxRequest())
            {
                return PartialView(tests);
            }

            return View(tests);
        }

        public ActionResult IndexAll(int? page, string order)
        {
            const int itemsOnPage = 10;
            int pg = page ?? 1;
            int totalItems;

            TestHelper.ClearTests(null);

            IList<Test> tests = new TestDao().GetTestsByOrderPaged(itemsOnPage, pg, out totalItems, order);

            ViewBag.CurrentPage = pg;
            ViewBag.ItemsOnPage = itemsOnPage;
            ViewBag.Order = order;

            if (Request.IsAjaxRequest())
            {
                return PartialView(tests);
            }
            return View(tests);
        }

        public JsonResult SearchRecords(string query)
        {

            IList<Test> tests = new TestDao().SearchTests(query);

            IList<string> responses = (from Test test in tests select test.Chapter.Name).ToList();

            IList<string> responses1 = new List<string>();

            foreach (string response in responses.Where(response => !responses1.Contains(response)))
            {
                responses1.Add(response);
            }

            return Json(responses1, JsonRequestBehavior.AllowGet);
        }

        public ActionResult Search(string phrase)
        {

            IList<Test> tests = new TestDao().SearchTests(phrase);

            if (Request.IsAjaxRequest())
            {
                return PartialView("IndexAll", tests);
            }
            return View("IndexAll", tests);
        }

        public ActionResult Detail(int id)
        {
            Test test = new TestDao().GetById(id);

            IList<QuestionsOfTest> questionsOfTests = new QuestionsOfTestDao().GetByTest(test);

            ViewBag.test = test;

            if (Request.IsAjaxRequest())
            {
                return PartialView(questionsOfTests);
            }
            return View(questionsOfTests);
        }

        public ActionResult Delete(int id)
        {
            TestDao testDao = new TestDao();
            Test test = testDao.GetById(id);
            AnswerDao answerDao = new AnswerDao();

            QuestionsOfTestDao questionsOfTestDao = new QuestionsOfTestDao();
            IList<QuestionsOfTest> questionsOfTests = questionsOfTestDao.GetByTest(test);

            if (questionsOfTests.Count != 0)
            {
                foreach (QuestionsOfTest questionsOfTest in questionsOfTests)
                {

                    IList<Answer> answers = answerDao.GetByQuestionsOfTestsAndType(questionsOfTest, 2);

                    foreach (Answer answer in answers)
                    {
                        answerDao.Delete(answer);
                    }

                    questionsOfTestDao.Delete(questionsOfTest);

                }
                testDao.Delete(test);
            }
            else
            {
                testDao.Delete(test);
            }

            TempData["message-success"] = "Test uživatele " + test.User.Name + " " + test.User.Surname + " byl úspěšně smazán";

            return RedirectToAction("IndexAll", "Test");
        }
    }
}
