﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Text;
using DataAccess.Dao;
using DataAccess.Model;

namespace PRIPO_v02.Class
{
    public class TestHelper
    {

        public static string RemoveDiacritics(string text)
        {
            string result = string.Concat(
                text.Normalize(NormalizationForm.FormD)
                .Where(ch => CharUnicodeInfo.GetUnicodeCategory(ch) != UnicodeCategory.NonSpacingMark)
                ).Normalize(NormalizationForm.FormC);
            return result.Replace(" ", string.Empty);
        }

        public static bool Compare(IList<Answer> rightAnswers, IList<Answer> answers, string type)
        {
            if (rightAnswers.Count == 0 && answers.Count == 0)
            {
                return true;
            }
            if (rightAnswers.Count != answers.Count && type != "Doplňovací")
            {
                return false;
            }

            int counter = 0;

            switch (type)
            {
                case "Zaškrtávací":
                    counter += rightAnswers.Sum(rightAnswer => answers.Count(answer => RemoveDiacritics(answer.Text).Equals(RemoveDiacritics(rightAnswer.Text))));
                    return rightAnswers.Count == counter;
                case "Doplňovací":
                    return (from rightAnswer in rightAnswers from answer in answers where string.Equals(RemoveDiacritics(answer.Text), RemoveDiacritics(rightAnswer.Text), StringComparison.CurrentCultureIgnoreCase) select rightAnswer).Any();
                default:
                    return false;
            }
        }

        public static void ClearTests(int? id)
        {
            TestDao testDao = new TestDao();

            var tests = id != null ? testDao.GetByUser(new UserDao().GetById((int)id)) : testDao.GetAll();

            foreach (Test test in tests)
            {
                if (test.Grade == 0f)
                {
                    AnswerDao answerDao = new AnswerDao();

                    QuestionsOfTestDao questionsOfTestDao = new QuestionsOfTestDao();
                    IList<QuestionsOfTest> questionsOfTests = questionsOfTestDao.GetByTest(test);

                    if (questionsOfTests.Count != 0)
                    {
                        foreach (QuestionsOfTest questionsOfTest in questionsOfTests)
                        {

                            IList<Answer> answers = answerDao.GetByQuestionsOfTestsAndType(questionsOfTest, 2);

                            foreach (Answer answer in answers)
                            {
                                answerDao.Delete(answer);
                            }

                            questionsOfTestDao.Delete(questionsOfTest);

                        }
                        testDao.Delete(test);
                    }
                    else
                    {
                        testDao.Delete(test);
                    }
                }
            }
        }

        public static float GetGrade(float result)
        {
            if (result >= 0.9)
            {
                return 1;
            }
            if (result >= 0.8 && result < 0.9)
            {
                return 1.5f;
            }
            if (result >= 0.7 && result < 0.8)
            {
                return 2;
            }
            if (result >= 0.6 && result < 0.7)
            {
                return 2.5f;
            }
            if (result >= 0.5 && result < 0.6)
            {
                return 3;
            }
            return 4;
        }
    }
}