﻿using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using DataAccess.Dao;
using DataAccess.Model;

namespace PRIPO_v02.Controllers
{
    public class ChapterController : Controller
    {
        public ActionResult Index(int? page)
        {
            const int itemsOnPage = 8;
            int pg = page ?? 1;
            int totalItems;

            ViewBag.CurrentPage = pg;
            ViewBag.ItemsOnPage = itemsOnPage;

            IList<Chapter> chapters = new ChapterDao().GetChaptersPaged(itemsOnPage, pg, out totalItems);

            if (Request.IsAjaxRequest())
            {
                return PartialView(chapters);
            }

            return View(chapters);
        }

        public JsonResult SearchRecords(string query)
        {

            IList<Chapter> chapters = new ChapterDao().SearchChapters(query);

            IList<string> responses = (from Chapter chapter in chapters select chapter.Name).ToList();

            IList<string> responses1 = new List<string>();

            foreach (string response in responses.Where(response => !responses1.Contains(response)))
            {
                responses1.Add(response);
            }

            return Json(responses1, JsonRequestBehavior.AllowGet);
        }

        public ActionResult Search(string phrase)
        {

            IList<Chapter> chapters = new ChapterDao().SearchChapters(phrase);

            if (Request.IsAjaxRequest())
            {
                return PartialView("Index", chapters);
            }

            return View("Index", chapters);
        }

        public ActionResult Detail(int id)
        {
            Chapter chapter = new ChapterDao().GetById(id);

            if (Request.IsAjaxRequest())
            {
                return PartialView(chapter);
            }

            return View(chapter);
        }

        public FileResult Download(int id)
        {
            Chapter chapter = new ChapterDao().GetById(id);

            var filepath = Path.Combine(Server.MapPath("~/Uploads/Presentations/"), chapter.Presentation);
            return File(filepath, MimeMapping.GetMimeMapping(filepath), chapter.Presentation);
        }
    }
}
